package com.diagbot.service.impl;

import com.diagbot.client.fastdfs.FastDFSClient;
import com.diagbot.client.fastdfs.FastDFSFile;
import com.diagbot.dto.FileDTO;
import com.diagbot.dto.FileThumDTO;
import com.diagbot.exception.CommonErrorCode;
import com.diagbot.exception.CommonException;
import com.diagbot.service.UploadService;
import lombok.extern.slf4j.Slf4j;
import net.coobird.thumbnailator.Thumbnails;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;

/**
 * @Description: 文件上传服务接口实现
 * @author: gaodm
 * @time: 2018/11/13 13:50
 */
@Slf4j
@Service
public class UploadServiceImpl implements UploadService {
    @Override
    public FileDTO singleFileUpload(MultipartFile file) {
        if (file.isEmpty()) {
            return new FileDTO("FAILURE", "文件不能为空");
        }
        //文件大小上限4M
        if (file.getSize() > 4 * 1024 * 1024) {
            return new FileDTO("FAILURE", "文件上传失败，超出大小限制4MB");
        }
        try {
            FileDTO fileDTO = saveFile(file);
            return fileDTO;
        } catch (Exception e) {
            log.error("文件上传失败", e);
            return new FileDTO("FAILURE", "文件上传失败，请重新上传");
        }
    }

    @Override
    public List<FileDTO> multiFileUpload(MultipartFile[] mpfs) {
        // 上传文件返回的路径集合
        List<FileDTO> fileDTOS = new ArrayList<>();
        if (null == mpfs) {
            throw new CommonException(CommonErrorCode.PARAM_IS_NULL, "文件不能为空");
        }

        for (MultipartFile file : mpfs) {
            if (file.isEmpty()) {
                fileDTOS.add(new FileDTO("FAILURE", "文件不能为空"));
                continue;
            }
            //文件大小上限4M
            if (file.getSize() > 4 * 1024 * 1024) {
                fileDTOS.add(new FileDTO("FAILURE", "文件上传失败，超出大小限制4MB"));
                continue;
            }

            try {
                FileDTO fileDTO = saveFile(file);
                fileDTOS.add(fileDTO);
            } catch (Exception e) {
                log.error("文件上传失败", e);
                fileDTOS.add(new FileDTO("FAILURE", "文件上传失败，请重新上传"));
            }
        }
        return fileDTOS;
    }


    @Override
    public List<FileThumDTO> multiFileThumUpload(MultipartFile[] mpfs) {
        // 上传文件返回的路径集合
        List<FileThumDTO> fileDTOS = new ArrayList<>();
        if (null == mpfs) {
            throw new CommonException(CommonErrorCode.PARAM_IS_NULL, "文件不能为空");
        }

        FileThumDTO fileThumDTO = null;
        for (MultipartFile file : mpfs) {
            if (file.isEmpty()) {
                fileThumDTO = new FileThumDTO();
                fileThumDTO.setSource(new FileDTO("FAILURE", "文件不能为空"));
                fileDTOS.add(fileThumDTO);
                continue;
            }
            //文件大小上限4M
            if (file.getSize() > 4 * 1024 * 1024) {
                fileThumDTO = new FileThumDTO();
                fileThumDTO.setSource(new FileDTO("FAILURE", "文件上传失败，超出大小限制4MB"));
                fileDTOS.add(fileThumDTO);
                continue;
            }

            try {
                fileThumDTO = saveFileWithThum(file);
                fileDTOS.add(fileThumDTO);
            } catch (Exception e) {
                log.error("文件上传失败", e);
                fileThumDTO = new FileThumDTO();
                fileThumDTO.setSource(new FileDTO("FAILURE", "文件上传失败，请重新上传"));
                fileDTOS.add(fileThumDTO);
            }
        }
        return fileDTOS;
    }

    /**
     * @param multipartFile
     * @return
     * @throws IOException
     */
    public FileThumDTO saveFileWithThum(MultipartFile multipartFile) throws IOException {
        FileThumDTO fileThumDTO = new FileThumDTO();
        //原图上传到服务器
        FileDTO source = saveFile(multipartFile);
        fileThumDTO.setSource(source);
        //压缩图片
        FileDTO thumbnail = saveFile(multipartFile, true);
        fileThumDTO.setThumbnail(thumbnail);
        return fileThumDTO;
    }

    /**
     * @param multipartFile
     * @return
     * @throws IOException
     */
    public FileDTO saveFile(MultipartFile multipartFile) throws IOException {
        return saveFile(multipartFile, false);
    }

    /**
     * @param multipartFile
     * @return
     * @throws IOException
     */
    public FileDTO saveFile(MultipartFile multipartFile, Boolean isThum) throws IOException {
        String[] fileAbsolutePath = {};
        String fileName = multipartFile.getOriginalFilename();
        String ext = fileName.substring(fileName.lastIndexOf(".") + 1);
        byte[] file_buff = null;
        InputStream inputStream = multipartFile.getInputStream();
        //压缩图片
        if (isThum) {
            Integer width = 280;
            Integer hight = 430;
            //文件名重新命名
            fileName = width + "_" + hight + "_" + fileName;
            inputStream = compress(inputStream, width, hight);
        }

        if (inputStream != null) {
            int len1 = inputStream.available();
            file_buff = new byte[len1];
            inputStream.read(file_buff);
        }
        inputStream.close();
        FastDFSFile file = new FastDFSFile(fileName, file_buff, ext);
        try {
            fileAbsolutePath = FastDFSClient.upload(file);  //upload to fastdfs
        } catch (Exception e) {
            log.error("文件上传异常", e);
            return new FileDTO("FAILURE", "文件上传异常");
        }
        if (fileAbsolutePath == null) {
            log.error("文件上传失败，请重新上传");
            return new FileDTO("FAILURE", "文件上传失败，请重新上传");
        }
        String path = "/" + fileAbsolutePath[0] + "/" + fileAbsolutePath[1];
        FileDTO fileDTO = new FileDTO("SUCCESS", "文件上传成功");
        fileDTO.setUrl(path);
        fileDTO.setOriginal(fileName);
        fileDTO.setTitle(fileName);
        return fileDTO;

    }

    public ByteArrayInputStream compress(InputStream inputStream, Integer width, Integer hight) throws IOException {
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        Thumbnails.of(inputStream)
                .size(width, hight)
                .toOutputStream(baos);
        final ByteArrayInputStream swapStream = new ByteArrayInputStream(baos.toByteArray());
        return swapStream;
    }

    /**
     * 删除服务端文件
     *
     * @param path
     * @return
     */
    public FileDTO deleteRemoteFile(String path) {
        if (path.startsWith("/")) {
            path = path.substring(1);
        }
        String fileName = path.substring(path.indexOf("/") + 1);
        String groupName = path.substring(0, path.indexOf("/"));
        try {
            FastDFSClient.deleteFile(groupName, fileName);
        } catch (Exception e) {
            log.error("", e);
            return new FileDTO("FAILURE", "文件删除失败");
        }
        return new FileDTO("SUCCESS", "文件删除成功");
    }
}